define(['dom', 'sandbox', '../collections/requests', './requests', 'text!../templates/base.html', 'text!../templates/no_pending_requests.html', 'core_lib/alerts'],
  function($, sandbox, RequestsCollection, RequestsView, template, no_requests_template, Alerts) {

  var Application = sandbox.mvc.Layout({

    template: template,

    regions: {
      requests_container: '#certificates-requests-container',
      no_requests_container: '#certificates-no-requests-container'
    },

    ui: {
      loading_message: '#certificates-loading-message',
      requests_container: '#certificates-requests-container',
      no_requests_container: '#certificates-no-requests-container'
    },

    initialize: function(options) {

      if (!options.skip_stylesheet) {
        sandbox.io.load_stylesheet('../applications/certificates/css/certificates.css');
      }

      this.requests = options.requests || new RequestsCollection;

      this.bindTo(this.requests, 'reset', this.show_list, this);

      var req = this.requests.fetch();

      req.fail($.proxy(function(request) {
        var content;

        switch (request.status) {
          case 403:
            content = 'There is a permissions problem with the <code>certificate_status</code> stanza of your ' +
              'certificate authority\'s auth.conf. See <a href="http://docs.puppetlabs.com/pe/2.7/console_cert_mgmt.html#configuration-details" target="_blank">this ' +
              'documentation</a> for details on resolving the problem.';
            break;
          default:
            content = 'An error occured trying to load the certificate requests. <a href="' + sandbox.paths.base_url + '">Reload</a> this page to try again.';
            break;
        }

        var alert = new Alerts.error({
          hide_close_button: true,
          content: content
        }).render();

        this.ui.loading_message.html(alert.el);

      }, this));
    },

    show_list: function() {

      this.ui.loading_message.hide();

      if (this.requests.length > 0) {

        this.requests_view = new RequestsView({
          collection: this.requests
        });

        this.requests_container.show(this.requests_view);

      } else {

        var view = new (sandbox.mvc.ItemView({
          template: no_requests_template
        }))();

        this.no_requests_container.show(view);
      }
    }

  });

  return Application;

});
