define(['jquery', 'sandbox', 'text!../templates/request.html'],
  function($, sandbox, template) {

  sandbox.template.engine.registerHelper('truncated_fingerprint', function() {
    var text = this.fingerprint,
        length = 12;

    if (!text) {
      return '';
    }

    return text.substr(0, length - 1) + (text.length > length ? '&hellip;' : '');
  });

  var THROBBER_IMG = '<img src="' + sandbox.paths.images + '/loading.gif" alt="Loading">';

  return sandbox.mvc.ItemView({

    tagName: 'tr',

    template: template,

    events: {
      'click .certificates-accept': 'accept_csr',
      'click .certificates-reject': 'reject_csr'
    },

    ui: {
      accept_button: '.certificates-accept',
      reject_button: '.certificates-reject',
      action_message: '.certificates-action-message',
      action_buttons: '.certificates-action-buttons',
      loading_indicator: '.certificates-loading-indicator'
    },

    initialize: function() {
      this.bindTo(this.model, 'change:status', this.render, this);
      this.bindTo(this.model, 'destroy', this.close, this);

      this.bindTo(this.model, 'sync:saving', function () {
        this.show_loading_indicator();
        this.hide_error();
      }, this);

      this.bindTo(this.model, 'sync:complete', this.hide_loading_indicator, this);

      this.bindTo(this.model, 'sync:failed', function() {
        this.hide_loading_indicator();

        this.show_error(this.model.get('message'));
      }, this);
    },

    destroy_tooltip: function() {
      this.$('a[rel="tooltip"]').tooltip('destroy');
      this.$('a[rel="popover"]').popover('destroy');
    },

    beforeClose: function() {
      this.destroy_tooltip();
    },

    beforeRender: function() {
      // remove any existing tooltips before we re-create the links that created them
      this.destroy_tooltip();
    },

    onRender: function() {
      this.$('a[rel="tooltip"]').tooltip();

      this.$('a.certificates-dns-altname-help[rel="popover"]').popover({
        placement: 'bottom',
        animation: false,
        html: true,
        title: 'DNS alt name',
        content: 'You cannot accept or reject requests in the console from nodes with a <a href="http://docs.puppetlabs.com/references/latest/configuration.html#dnsaltnames" target="_blank">DNS alt name</a>. ' +
                 'Run <code>puppet cert --allow-dns-alt-names sign ' + this.model.get('name') + '</code> on the CA to accept it'
      });

      this.$el.addClass(this.model.get('approvable') ? 'certificate-valid-row' : 'certificate-invalid-row');
    },

    accept_csr: function() {
      this.model.accept();
    },

    reject_csr: function() {
      this.model.reject();
    },

    show_loading_indicator: function() {
      this.ui.loading_indicator.html(THROBBER_IMG);
      this.ui.action_buttons.hide();
    },

    hide_loading_indicator: function() {
      this.ui.loading_indicator.html('');
      this.ui.action_buttons.show();
    },

    /**
     * Display an inline error message with a popup that contains the full error text
     * @param error {string}
     */
    show_error: function(error) {
      this.$('a.certificates-error-description[rel="popover"]').popover('destroy');

      var popup = $('<a class="certificates-error-description" rel="popover"></a>')
                    .html('Could not process this request <img src="' + sandbox.paths.images + '/help-icon.png" alt="' + error + '">')
                    .popover({
                      placement: 'bottom',
                      animation: false,
                      title: 'An error occured',
                      content: error
                    });

      this.ui.action_message.html(popup).show();
    },

    hide_error: function() {
      this.$('a.certificates-error-description[rel="popover"]').popover('destroy');

      this.ui.action_message.html('').hide();
    }
  });
});